/*****************************************************************************
 * Copyright 2012 celum Slovakia s r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package com.celum.dbplugin.resource;

import com.celum.dbplugin.installer.SqlScript;
import com.celum.dbplugin.installer.SqlScriptsResource;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Common abstract class that is responsible for filtering SQL scripts based
 * on DB version.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public abstract class VersionFilter extends SqlScriptsResource
{
	//-------------------------------------------------------------------------------------------------------------------
	// members & constants
	//-------------------------------------------------------------------------------------------------------------------

	/** referring to real sql script resource */
	private final SqlScriptsResource decoratedResource;

	/** pattern is applyed on script's ID and is used for version extraction */
	private final Pattern stepPattern;


	//-------------------------------------------------------------------------------------------------------------------
	// abstr. methods (for implementation)
	//-------------------------------------------------------------------------------------------------------------------


	/**
	 * Here you will implement your code that returns the
	 * current DB version.
	 *
	 * @return
	 */
	abstract int getDbVersion();

	//-------------------------------------------------------------------------------------------------------------------
	// constructor & methods
	//-------------------------------------------------------------------------------------------------------------------

	/**
	 * Constructor
	 * @param decoratedResource
	 * @param stepPattern
	 */
	protected VersionFilter(SqlScriptsResource decoratedResource, String stepPattern)
	{
		this.decoratedResource = decoratedResource;
		if (stepPattern == null) {
			this.stepPattern = Pattern.compile("(\\d+)");
		} else {
			this.stepPattern = Pattern.compile(stepPattern);
		}
	}


	@Override
	final public Collection<SqlScript> getSortedScripts()
	{
		Collection<SqlScript> scripts = decoratedResource.getSortedScripts();
		int version = getDbVersion();
		scripts = filterLaterScripts(scripts, version);
		return scripts;
	}


	final Collection<SqlScript> filterLaterScripts(Collection<SqlScript> scripts, int currentVersion)
	{
		List<SqlScript> out = new LinkedList<SqlScript>();
		for (SqlScript script : scripts) {
			Matcher matcher = stepPattern.matcher(script.getId());
			if (matcher.find()) {
				int scriptVersion = Integer.parseInt(matcher.group(1));
				boolean scriptIsLater = scriptVersion > currentVersion;
				if (scriptIsLater) {
					out.add(script);
				}
			}
		}

		Collections.sort(out);
		return out;
	}

}
